import pyglet
from pyglet import font
from pyglet.gl import *
from pyglet.window import key
import time
import math
import shapes

versionString = 'Ver. 20110714'

# ===========================================================================
updateInterval = 1 # Update currently relies on this being 1
totalSeconds = 0

# ===========================================================================
winWidth = 300
winHeight = 120

win = pyglet.window.Window(width = winWidth, height = winHeight, caption = 'Webpage: taivo.net', visible = True)
icon = pyglet.image.load('TL_MiniTime_Icon_128.png')
win.set_icon(icon)

backgroundColor = [0, 0, 0, 1]
glClearColor(*backgroundColor)

# ===========================================================================
timeText = font.Text(
  font.load('Helvetica', 24),
  '00:00:00',
  x = win.width / 2, y = win.height / 2,
  halign = font.Text.CENTER, valign = font.Text.CENTER,
  color = (0, 1, 0, 1))

totalTimeText = font.Text(
  font.load('Helvetica', 12),
  '00:00:00',
  x = win.width / 2, y = 9,
  halign = font.Text.CENTER, valign = font.Text.BASELINE,
  color = (1, 0, 0, 1))

def generateHelpTextList(fontSize, startingY, repeatY):
  return [font.Text(
            font.load('Verdana', fontSize, bold = True),
            line,
            x = 4, y = lineY,
            halign = font.Text.LEFT, valign = font.Text.BASELINE,
            color = (0, 1, 0, 1))
          for line, lineY in zip(
            ['Esc -- quit',
             'h -- help',
             'r -- reset green',
             'Shift + c -- Clear all',
             'Space or click -- run / pause',
             '1/2/3 -- normal/large/extra large'],
            range(startingY, startingY + 6 * repeatY, repeatY))]

lineHeight = 19
helpTextList = generateHelpTextList(10, 6, lineHeight)

appNameText = font.Text(
  font.load('Helvetica', 12, bold = True),
  'TL MiniTime',
  x = win.width - 2, y = 6 + 2 * lineHeight,
  halign = font.Text.RIGHT, valign = font.Text.BASELINE,
  color = (0, .7, .7, 1))

versionText = font.Text(
  font.load('Helvetica', 12, bold = True),
  versionString,
  x = win.width - 2, y = 6 + lineHeight,
  halign = font.Text.RIGHT, valign = font.Text.BASELINE,
  color = (0, .7, .7, 1))

emailText = font.Text(
  font.load('Verdana', 10, bold = True),
  'minitime@taivo.net',
  x = win.width - 2, y = 6,
  halign = font.Text.RIGHT, valign = font.Text.BASELINE,
  color = (.85, .85, 0, 1))

pressHelpText = font.Text(
  font.load('Helvetica', 13),
  'Press h for help',
  x = 2, y = win.height - 2,
  halign = font.Text.LEFT, valign = font.Text.TOP,
  color = (1, 1, 0, 1))

displayPressHelpText = True
def stopDisplayingPressHelpText(dt):
  global displayPressHelpText
  displayPressHelpText = False

# ===========================================================================
def secondsToHMS(totalSeconds):
  hours, minSecSeconds = divmod(totalSeconds, 3600)
  minutes, seconds = divmod(minSecSeconds, 60)
  seconds = round(seconds)
  return map(int, [hours, minutes, seconds])

def formattedHMSstring(hours, minutes, seconds):
  h = str(hours).rjust(2, '0')
  m = str(minutes).rjust(2, '0')
  s = str(seconds).rjust(2, '0')
  return h + ':' + m + ':' + s

# ===========================================================================
elapsedBeforePause = 0
pause = True
oneShotScheduled = False
wasReset = False

def updateTime(dt):
  global pause
  global oneShotScheduled
  global wasReset

  if not pause:
    elapsedSeconds = time.time() - startTime + elapsedBeforePause

    if wasReset and elapsedSeconds < 1:
      pass
    else:
      timeText.text = formattedHMSstring(*secondsToHMS(elapsedSeconds))
      totalTimeText.text = formattedHMSstring(*secondsToHMS(totalSeconds + elapsedSeconds))
    
    if wasReset:
      wasReset = False

    syncDif = elapsedSeconds - round(elapsedSeconds) 
    if syncDif > .01:
      pyglet.clock.unschedule(updateTime)
      pyglet.clock.schedule_once(updateTime, 1 - syncDif)
      oneShotScheduled = True
    elif syncDif < -.01:
      pyglet.clock.unschedule(updateTime)
      pyglet.clock.schedule_once(updateTime, abs(syncDif))
      oneShotScheduled = True
    elif oneShotScheduled:
      pyglet.clock.schedule_interval(updateTime, updateInterval)
      oneShotScheduled = False

    #print elapsedSeconds, totalSeconds + elapsedSeconds

updateTime(0)

def togglePause():
  global pause
  global startTime
  global elapsedBeforePause
  global oneShotScheduled

  if pause:
    startTime = time.time()
    pause = False
    oneShotScheduled = True
    pyglet.clock.schedule_once(updateTime, math.ceil(elapsedBeforePause) - elapsedBeforePause)
  else:
    elapsedBeforePause += time.time() - startTime
    pyglet.clock.unschedule(updateTime)
    pause = True

# ===========================================================================
help = False

@win.event
def on_key_press(symbol, modifiers):

  global pause
  global startTime
  global elapsedBeforePause
  global totalSeconds
  global pRectX, pRectY, pRectWx, pRectWy 
  global pRectLineWidth
  global help
  global wasReset
  global oneShotScheduled

  if help:
    help = False
    return True

  if symbol == key.H:
    help = not help

  if symbol == key.SPACE:
    togglePause()

  if symbol == key.R:
    if pause:
      totalSeconds += elapsedBeforePause
    else:
      wasReset = True
      pyglet.clock.unschedule(updateTime)
      totalSeconds += elapsedBeforePause + time.time() - startTime 
      startTime = time.time()
      pyglet.clock.schedule_interval(updateTime, updateInterval)
    elapsedBeforePause = 0
    timeText.text = '00:00:00'
  
  if symbol == key.C and modifiers & key.MOD_SHIFT:
    if not pause:
      wasReset = True
      #print pyglet.clock.get_sleep_time(True)
      pyglet.clock.unschedule(updateTime)
      #print pyglet.clock.get_sleep_time(True)
      startTime = time.time()
      oneShotScheduled = True
      pyglet.clock.schedule_once(updateTime, 1)
    timeText.text = '00:00:00'
    totalTimeText.text = '00:00:00'
    elapsedBeforePause = 0
    totalSeconds = 0

  if symbol == key._1:
    win.set_size(winWidth, winHeight)
    timeText.font = font.load('Helvetica', 24)
    totalTimeText.font = font.load('Helvetica', 12)
    totalTimeText.y = 9
    pRectX, pRectY, pRectWx, pRectWy = 218, 70, 3, 3
    pRectLineWidth = 1

  if symbol == key._2:
    win.set_size(winWidth * 2, winHeight * 2)
    timeText.font = font.load('Helvetica', 64)
    totalTimeText.font = font.load('Helvetica', 24)
    totalTimeText.y = 16
    pRectX, pRectY, pRectWx, pRectWy = 479, 148, 6, 6
    pRectLineWidth = 2

  if symbol == key._3:
    win.set_size(winWidth * 3, winHeight * 3)
    timeText.font = font.load('Helvetica', 148)
    totalTimeText.font = font.load('Helvetica', 52)
    totalTimeText.y = 20 
    pRectX, pRectY, pRectWx, pRectWy = 858, 252, 10, 10
    pRectLineWidth = 3

pRectX, pRectY, pRectWx, pRectWy = 218, 70, 3, 3
pRectLineWidth = 1

@win.event
def on_draw():
  glClear(GL_COLOR_BUFFER_BIT)

  if help:
    for t in helpTextList:
      t.draw()
    appNameText.draw()
    versionText.draw()
    emailText.draw()
  else:
    timeText.draw()
    totalTimeText.draw()
    if pause:
      for i in range(pRectLineWidth):
        shapes.drawRectangle(pRectX + i, pRectY + i, pRectWx - 2 * i, pRectWy - 2 * i, [1, 1, 0, 1], False)

  if displayPressHelpText:
    pressHelpText.draw()

@win.event
def on_mouse_press(x, y, button, modifiers):
  global help
  if help:
    help = False
    return

  togglePause()

@win.event
def on_resize(width, height):
  timeText.x = width / 2
  timeText.y = height / 2
  totalTimeText.x = width / 2

# ===========================================================================
pyglet.clock.schedule_once(stopDisplayingPressHelpText, 1.2)
# ===========================================================================
pyglet.app.run()

win.close()

